﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using CascadingDropdowns.Service;

namespace CascadingDropdowns.Models
{
    public class ProductFilter
    {
        static ProductService service;

        public static ProductService Service
        {
            get
            {
                if (service == null)
                {
                    service = new ProductService();
                }
                return service;
            }
        }

        public static IEnumerable<SelectListItem> GetProductTypes()
        {
            return Service.ProductTypes.ToList().Select(
                pt => new SelectListItem() { Value = pt.Id.ToString(), Text = pt.Name });
        }

        public static IEnumerable<SelectListItem> GetManufacturers(int? productTypeId)
        {
            return Service.Manufacturers.Where(
                m => !productTypeId.HasValue ||
                    (productTypeId.HasValue && m.Models.Any(model => model.Type.Id == productTypeId.Value))).ToList()
                    .Select(m => new SelectListItem() { Value = m.Id.ToString(), Text = m.Name });
        }

        public Manufacturer Manufacturer { get; set; }

        public Model Model { get; set; }

        public ProductType ProductType { get; set; }

        public ProductVersion ProductVersion { get; set; }

        public TrimLevel TrimLevel { get; set; }
    }
}